@@ -1,6 +1,6 @@
 /*
  * Copyright © 2005 Novell, Inc.
- * Copyright (C) 2007, 2008 Kristian Lyngstøl
+ * Copyright (C) 2007, 2008,2010 Kristian Lyngstøl
  *
  * Permission to use, copy, modify, distribute, and sell this software
  * and its documentation for any purpose is hereby granted without
@@ -23,9 +23,10 @@
  *
  *
  * Author(s):
- *	- Original zoom plug-in; David Reveman <davidr@novell.com>
  *	- Most features beyond basic zoom;
  *	  Kristian Lyngstol <kristian@bohemians.org>
+ *	- Original zoom plug-in; David Reveman <davidr@novell.com>
+ *	- Original port to C++ by Sam Spilsbury <smspillaz@gmail.com>
  *
  * Description:
  *
@@ -59,12 +60,15 @@
  *
  * This second method has one huge issue, which is bugged XFixes. After
  * hiding the cursor once with XFixes, some mouse cursors will simply be
- * invisible. The Firefox loading cursor being one of them. 
+ * invisible. The Firefox loading cursor being one of them.
  *
  * An other minor annoyance is that mouse sensitivity seems to increase as
  * you zoom in, since the mouse isn't really zoomed at all.
  *
  * Todo:
+ *  - Walk through C++ port and adjust comments for 2010.
+ *  - See if anyone misses the filter setting
+ *  - Verify XFixes fix... err.
  *  - Different multi head modes
  */
 
@@ -73,29 +77,19 @@
 COMPIZ_PLUGIN_20090315 (ezoom, ZoomPluginVTable)
 
 
-/* This toggles paint functions. We don't need to continually run code when we
- * are not doing anything */
+/*
+ * This toggles paint functions. We don't need to continually run code when we
+ * are not doing anything
+ */
 static inline void
-toggleFunctions (bool enabled)
+toggleFunctions (bool state)
 {
     ZOOM_SCREEN (screen);
 
-    screen->handleEventSetEnabled (zs, enabled);
-    zs->cScreen->preparePaintSetEnabled (zs, enabled);
-    zs->gScreen->glPaintOutputSetEnabled (zs, enabled);
-    zs->cScreen->donePaintSetEnabled (zs, enabled);
-}
-
-/* Checks if a specific screen grab exist. DO NOT USE THIS.
- * This is a temporary fix that SHOULD be removed asap.
- * See comments in drawCursor.
- */
-static inline bool
-dontuseScreengrabExist (char * grab)
-{
-    if (screen->otherGrabExist (grab, NULL))
-	return true;
-    return false;
+    screen->handleEventSetEnabled (zs, state);
+    zs->cScreen->preparePaintSetEnabled (zs, state);
+    zs->gScreen->glPaintOutputSetEnabled (zs, state);
+    zs->cScreen->donePaintSetEnabled (zs, state);
 }
 
 /* Check if the output is valid */
@@ -104,8 +98,10 @@ outputIsZoomArea (int out)
 {
     ZOOM_SCREEN (screen);
 
-    if (out < 0 || (unsigned int) out >= zs->zooms.size ())
+    if (out < 0)
 	return false;
+    else if ((unsigned int) out >= zs->zooms.size ())
+	zs->zooms.resize (screen->outputDevs ().size ());
     return true;
 }
 
@@ -133,7 +129,7 @@ isZoomed (int out)
     if (!outputIsZoomArea (out))
 	return false;
 
-    if (zs->zooms.at (out).currentZoom != 1.0f 
+    if (zs->zooms.at (out).currentZoom != 1.0f
 	|| zs->zooms.at (out).newZoom != 1.0f)
 	return true;
 
@@ -152,11 +148,11 @@ EZoomScreen::distanceToEdge (int out, EZoomScreen::ZoomEdge edge)
 
     if (!isActive (out))
 	return 0;
-    convertToZoomedTarget (out, o->region ()->extents.x2, 
+    convertToZoomedTarget (out, o->region ()->extents.x2,
 			   o->region ()->extents.y2, &x2, &y2);
-    convertToZoomedTarget (out, o->region ()->extents.x1, 
+    convertToZoomedTarget (out, o->region ()->extents.x1,
 			   o->region ()->extents.y1, &x1, &y1);
-    switch (edge) 
+    switch (edge)
     {
 	case NORTH: return o->region ()->extents.y1 - y1;
 	case SOUTH: return y2 - o->region ()->extents.y2;
@@ -282,12 +278,12 @@ EZoomScreen::adjustXYVelocity (int out, float chunk)
 
     if (xamount < 1.0f)
 	    xamount = 1.0f;
-    else if (xamount > 5.0) 
+    else if (xamount > 5.0)
 	    xamount = 5.0f;
-    
-    if (yamount < 1.0f) 
+
+    if (yamount < 1.0f)
 	    yamount = 1.0f;
-    else if (yamount > 5.0) 
+    else if (yamount > 5.0)
 	    yamount = 5.0f;
 
     zooms.at (out).xVelocity =
@@ -349,7 +345,7 @@ EZoomScreen::preparePaint (int	   msSinceLastPaint)
 		}
 	    }
 	}
-	if (optionGetSyncMouse ())
+	if (optionGetZoomMode () == EzoomOptions::ZoomModeSyncMouse)
 	    syncCenterToMouse ();
     }
 
@@ -381,7 +377,7 @@ EZoomScreen::donePaint ()
 }
 /* Draws a box from the screen coordinates inx1,iny1 to inx2,iny2 */
 void
-EZoomScreen::drawBox (const GLMatrix &transform, 
+EZoomScreen::drawBox (const GLMatrix &transform,
 		     CompOutput          *output,
 		     CompRect             box)
 {
@@ -393,7 +389,7 @@ EZoomScreen::drawBox (const GLMatrix &transform,
     zTransform.toScreenSpace (output, -DEFAULT_Z_CAMERA);
     convertToZoomed (out, box.x1 (), box.y1 (), &inx1, &iny1);
     convertToZoomed (out, box.x2 (), box.y2 (), &inx2, &iny2);
-    
+
     x1 = MIN (inx1, inx2);
     y1 = MIN (iny1, iny2);
     x2 = MAX (inx1, inx2);
@@ -432,7 +428,6 @@ EZoomScreen::glPaintOutput (const GLScreenPaintAttrib &attrib,
     if (isActive (out))
     {
 	GLScreenPaintAttrib sa = attrib;
-	int		    saveFilter;
 	GLMatrix            zTransform = transform;
 
 	mask &= ~PAINT_SCREEN_REGION_MASK;
@@ -443,21 +438,14 @@ EZoomScreen::glPaintOutput (const GLScreenPaintAttrib &attrib,
 		          1.0f);
 	zTransform.translate (zooms.at (out).xtrans,
 			      zooms.at (out).ytrans,
-			      0); 
+			      0);
 
 	mask |= PAINT_SCREEN_TRANSFORMED_MASK;
-	saveFilter = gScreen->textureFilter ();
-
-	if (optionGetFilterLinear ())
-	    gScreen->setTextureFilter (GLTexture::Good);
-	else
-	    gScreen->setTextureFilter (GLTexture::Fast);
 
 	status = gScreen->glPaintOutput (sa, zTransform, region, output, mask);
 
 	drawCursor (output, transform);
 
-	gScreen->setTextureFilter (saveFilter);
     }
     else
     {
@@ -530,17 +518,17 @@ EZoomScreen::setCenter (int x, int y, bool instant)
 	zooms.at (out).updateActualTranslates ();
     }
 
-    if (optionGetMousePan ())
+    if (optionGetZoomMode () == EzoomOptions::ZoomModePanArea)
 	restrainCursor (out);
 }
 
 /* Zooms the area described.
  * The math could probably be cleaned up, but should be correct now. */
 void
-EZoomScreen::setZoomArea (int        x, 
-	     		 int        y, 
-	     		 int        width, 
-	     		 int        height, 
+EZoomScreen::setZoomArea (int        x,
+	     		 int        y,
+	     		 int        width,
+	     		 int        height,
 	     		 bool       instant)
 {
     CompWindow::Geometry outGeometry (x, y, width, height, 0);
@@ -569,7 +557,7 @@ EZoomScreen::setZoomArea (int        x,
 	zooms.at (out).updateActualTranslates ();
     }
 
-    if (optionGetMousePan ())
+    if (optionGetZoomMode () == EzoomOptions::ZoomModePanArea)
 	restrainCursor (out);
 }
 
@@ -581,7 +569,7 @@ EZoomScreen::areaToWindow (CompWindow *w)
     int width = w->width () + w->input ().left + w->input ().right;
     int top = w->serverY () - w->input ().top;
     int height = w->height ()  + w->input ().top + w->input ().bottom;
-    
+
     setZoomArea (left, top, width, height, false);
 }
 
@@ -616,8 +604,8 @@ EZoomScreen::enableMousePolling ()
     mouse = MousePoller::getCurrentPosition ();
 }
 
-/* Sets the zoom (or scale) level. 
- * Cleans up if we are suddenly zoomed out. 
+/* Sets the zoom (or scale) level.
+ * Cleans up if we are suddenly zoomed out.
  */
 void
 EZoomScreen::setScale (int out, float value)
@@ -632,7 +620,7 @@ EZoomScreen::setScale (int out, float value)
 	if (!pollHandle.active ())
 	    enableMousePolling ();
 	grabbed |= (1 << zooms.at (out).output);
-	cursorZoomActive ();
+	cursorZoomActive (out);
     }
 
     if (value == 1.0f)
@@ -649,7 +637,7 @@ EZoomScreen::setScale (int out, float value)
     cScreen->damageScreen();
 }
 
-/* Sets the zoom factor to the bigger of the two floats supplied. 
+/* Sets the zoom factor to the bigger of the two floats supplied.
  * Convenience function for setting the scale factor for an area.
  */
 static inline void
@@ -661,7 +649,7 @@ setScaleBigger (int out, float x, float y)
 
 /* Mouse code...
  * This takes care of keeping the mouse in sync with the zoomed area and
- * vice versa. 
+ * vice versa.
  * See heading for description.
  */
 
@@ -673,7 +661,7 @@ void
 EZoomScreen::syncCenterToMouse ()
 {
     int         x, y;
-    int         out; 
+    int         out;
     CompOutput  *o;
 
     out = screen->outputDeviceForPoint (mouse.x (), mouse.y ());
@@ -698,13 +686,21 @@ EZoomScreen::syncCenterToMouse ()
 
 /* Convert the point X,Y to where it would be when zoomed.  */
 void
-EZoomScreen::convertToZoomed (int        out, 
-			     int        x, 
-			     int        y, 
-			     int        *resultX, 
+EZoomScreen::convertToZoomed (int        out,
+			     int        x,
+			     int        y,
+			     int        *resultX,
 			     int        *resultY)
 {
-    CompOutput  *o = &screen->outputDevs ()[out];
+    CompOutput *o;
+
+    if (!outputIsZoomArea (out))
+    {
+	*resultX = x;
+	*resultY = y;
+    }
+
+    o = &screen->outputDevs ()[out];
     ZoomArea    &za = zooms.at (out);
 
     x -= o->x1 ();
@@ -729,7 +725,14 @@ EZoomScreen::convertToZoomedTarget (int	  out,
 			           int	  *resultX,
 			           int	  *resultY)
 {
-    CompOutput  *o = &screen->outputDevs ().at (out);
+    CompOutput *o = &screen->outputDevs ().at (out);
+
+    if (!outputIsZoomArea (out))
+    {
+	*resultX = x;
+	*resultY = y;
+    }
+
     ZoomArea    &za = zooms.at (out);
 
     x -= o->x1 ();
@@ -794,7 +797,7 @@ EZoomScreen::ensureVisibility (int x, int y, int margin)
  * See ensureVisibility () for details.
  *
  * This attempts to find the translations that leaves the biggest part of
- * the area visible. 
+ * the area visible.
  *
  * gravity defines what part of the window that should get
  * priority if it isn't possible to fit all of it.
@@ -808,9 +811,9 @@ EZoomScreen::ensureVisibilityArea (int         x1,
 				  ZoomGravity gravity)
 {
     int        targetX, targetY, targetW, targetH;
-    int        out; 
-    CompOutput *o; 
-    
+    int        out;
+    CompOutput *o;
+
     out = screen->outputDeviceForPoint (x1 + (x2-x1/2), y1 + (y2-y1/2));
     o = &screen->outputDevs ().at (out);
 
@@ -829,13 +832,13 @@ EZoomScreen::ensureVisibilityArea (int         x1,
 	case NORTHWEST:
 	    targetX = x1;
 	    targetY = y1;
-	    if (WIDTHOK) 
+	    if (WIDTHOK)
 		targetW = x2 - x1;
-	    else 
+	    else
 		targetW = o->width () * zooms.at (out).newZoom;
-	    if (HEIGHTOK) 
+	    if (HEIGHTOK)
 		targetH = y2 - y1;
-	    else 
+	    else
 		targetH = o->height () * zooms.at (out).newZoom;
 	    break;
 	case NORTHEAST:
@@ -844,7 +847,7 @@ EZoomScreen::ensureVisibilityArea (int         x1,
 	    {
 		targetX = x1;
 		targetW = x2-x1;
-	    } 
+	    }
 	    else
 	    {
 		targetX = x2 - o->width () * zooms.at (out).newZoom;
@@ -853,7 +856,7 @@ EZoomScreen::ensureVisibilityArea (int         x1,
 
 	    if (HEIGHTOK)
 		targetH = y2-y1;
-	    else 
+	    else
 		targetH = o->height () * zooms.at (out).newZoom;
 	    break;
 	case SOUTHWEST:
@@ -866,7 +869,7 @@ EZoomScreen::ensureVisibilityArea (int         x1,
 	    {
 		targetY = y1;
 		targetH = y2-y1;
-	    } 
+	    }
 	    else
 	    {
 		targetY = y2 - (o->width () * zooms.at (out).newZoom);
@@ -878,13 +881,13 @@ EZoomScreen::ensureVisibilityArea (int         x1,
 	    {
 		targetX = x1;
 		targetW = x2-x1;
-	    } 
-	    else 
+	    }
+	    else
 	    {
 		targetW = o->width () * zooms.at (out).newZoom;
 		targetX = x2 - targetW;
 	    }
-	    
+
 	    if (HEIGHTOK)
 	    {
 		targetY = y1;
@@ -933,11 +936,11 @@ EZoomScreen::restrainCursor (int out)
 	mouse = MousePoller::getCurrentPosition ();
     }
 
-    convertToZoomedTarget (out, mouse.x () - cursor.hotX, 
+    convertToZoomedTarget (out, mouse.x () - cursor.hotX,
 			   mouse.y () - cursor.hotY, &x1, &y1);
-    convertToZoomedTarget 
-	(out, 
-	 mouse.x () - cursor.hotX + cursor.width, 
+    convertToZoomedTarget
+	(out,
+	 mouse.x () - cursor.hotX + cursor.width,
 	 mouse.y () - cursor.hotY + cursor.height,
 	 &x2, &y2);
 
@@ -951,13 +954,13 @@ EZoomScreen::restrainCursor (int out)
 
     if (y2 > o->y2 () - margin && south > 0)
 	diffY = y2 - o->y2 () + margin;
-    else if (y1 < o->y1 () + margin && north > 0) 
+    else if (y1 < o->y1 () + margin && north > 0)
 	diffY = y1 - o->y1 () - margin;
 
     if (abs(diffX)*z > 0  || abs(diffY)*z > 0)
 	screen->warpPointer ((int) (mouse.x () - pointerX) -
 						       (int) ((float)diffX * z),
-		     	     (int) (mouse.y () - pointerY) -  
+		     	     (int) (mouse.y () - pointerY) -
 						      (int) ((float)diffY * z));
 }
 
@@ -978,19 +981,19 @@ EZoomScreen::cursorMoved ()
 	if (optionGetRestrainMouse ())
 	    restrainCursor (out);
 
-	if (optionGetMousePan ())
+	if (optionGetZoomMode () == EzoomOptions::ZoomModePanArea)
 	{
 	    ensureVisibilityArea (mouse.x () - cursor.hotX,
 				  mouse.y () - cursor.hotY,
 				  mouse.x () + cursor.width -
 				  cursor.hotX,
-				  mouse.y () + cursor.height - 
+				  mouse.y () + cursor.height -
 				  cursor.hotY,
 				  optionGetRestrainMargin (),
 				  NORTHWEST);
 	}
 
-	cursorZoomActive ();
+	cursorZoomActive (out);
     }
     else
     {
@@ -1005,12 +1008,13 @@ EZoomScreen::cursorMoved ()
 void
 EZoomScreen::updateMousePosition (const CompPoint &p)
 {
-    int out; 
+    int out;
     mouse.setX (p.x ());
     mouse.setY (p.y ());
     out = screen->outputDeviceForPoint (mouse.x (), mouse.y ());
     lastChange = time(NULL);
-    if (optionGetSyncMouse () && !isInMovement (out))
+    if (optionGetZoomMode () == EzoomOptions::ZoomModeSyncMouse &&
+        !isInMovement (out))
 	setCenter (mouse.x (), mouse.y (), true);
     cursorMoved ();
     cScreen->damageScreen ();
@@ -1037,8 +1041,7 @@ EZoomScreen::freeCursor (CursorTexture * cursor)
 {
     if (!cursor->isSet)
 	return;
-	
-    //makeScreenCurrent (cursor->screen); ??
+
     cursor->isSet = false;
     glDeleteTextures (1, &cursor->texture);
     cursor->texture = 0;
@@ -1046,7 +1049,7 @@ EZoomScreen::freeCursor (CursorTexture * cursor)
 
 /* Translate into place and draw the scaled cursor.  */
 void
-EZoomScreen::drawCursor (CompOutput          *output, 
+EZoomScreen::drawCursor (CompOutput          *output,
 	    		const GLMatrix      &transform)
 {
     int         out = output->id ();
@@ -1055,16 +1058,13 @@ EZoomScreen::drawCursor (CompOutput          *output,
     {
 	GLMatrix      sTransform = transform;
 	float	      scaleFactor;
-	int           ax, ay, x, y;	
+	int           ax, ay, x, y;
 
-	/* FIXME:
-	 * This is a hack because these transformations are wrong when
-	 * we're working exposed. Expo is capable of telling where the
-	 * real mouse is despite zoom, so we
- don't have to disable the
-	 * zoom. We do, however, have to show the original pointer.
+	/*
+	 * XXX: expo knows how to handle mouse when zoomed, so we back off
+	 * when expo is active.
 	 */
-	if (dontuseScreengrabExist ((char *) "expo"))
+	if (screen->grabExist ( "expo"))
 	{
 	    cursorZoomInactive ();
 	    return;
@@ -1075,9 +1075,9 @@ EZoomScreen::drawCursor (CompOutput          *output,
         glPushMatrix ();
 	glLoadMatrixf (sTransform.getMatrix ());
 	glTranslatef ((float) ax, (float) ay, 0.0f);
-	if (optionGetScaleMouseDynamic ()) 
+	if (optionGetScaleMouseDynamic ())
 	    scaleFactor = 1.0f / zooms.at (out).currentZoom;
-	else 
+	else
 	    scaleFactor = 1.0f / optionGetScaleMouseStatic ();
 	glScalef (scaleFactor,
 		  scaleFactor,
@@ -1119,45 +1119,19 @@ EZoomScreen::updateCursor (CursorTexture * cursor)
     {
 	cursor->isSet = true;
 	cursor->screen = screen;
-	// makeScreenCurrent (s); // ???
 	glEnable (GL_TEXTURE_RECTANGLE_ARB);
 	glGenTextures (1, &cursor->texture);
 	glBindTexture (GL_TEXTURE_RECTANGLE_ARB, cursor->texture);
 
-	if (optionGetFilterLinear () &&
-	    gScreen->textureFilter () != GL_NEAREST) // ???
-	{
-	    glTexParameteri (GL_TEXTURE_RECTANGLE_ARB,
-			     GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-	    glTexParameteri (GL_TEXTURE_RECTANGLE_ARB,
-			     GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-	}
-	else
-	{
-	    glTexParameteri (GL_TEXTURE_RECTANGLE_ARB,
-			     GL_TEXTURE_MIN_FILTER, GL_NEAREST);
-	    glTexParameteri (GL_TEXTURE_RECTANGLE_ARB,
-			     GL_TEXTURE_MAG_FILTER, GL_NEAREST);
-	}
 	glTexParameteri (GL_TEXTURE_RECTANGLE_ARB,
 			 GL_TEXTURE_WRAP_S, GL_CLAMP);
 	glTexParameteri (GL_TEXTURE_RECTANGLE_ARB,
 			 GL_TEXTURE_WRAP_T, GL_CLAMP);
     } else {
-	//makeScreenCurrent (cursor->screen); ???
 	glEnable (GL_TEXTURE_RECTANGLE_ARB);
     }
 
     XFixesCursorImage *ci = XFixesGetCursorImage(dpy);
-    /* Hack to avoid changing to an invisible (bugged)cursor image.
-     * Example: The animated Firefox cursors.
-     */
-    if (ci->width <= 1 && ci->height <= 1)
-    {
-	XFree (ci);
-	return;
-    }
-
 
     cursor->width = ci->width;
     cursor->height = ci->height;
@@ -1165,7 +1139,7 @@ EZoomScreen::updateCursor (CursorTexture * cursor)
     cursor->hotY = ci->yhot;
     pixels = (unsigned char *) malloc (ci->width * ci->height * 4);
 
-    if (!pixels) 
+    if (!pixels)
     {
 	XFree (ci);
 	return;
@@ -1220,11 +1194,19 @@ EZoomScreen::cursorZoomInactive ()
  * activation.
  */
 void
-EZoomScreen::cursorZoomActive ()
+EZoomScreen::cursorZoomActive (int out)
 {
     if (!fixesSupported)
 	return;
-    if (!optionGetScaleMouse ())
+
+    /* Force cursor hiding and mouse panning if this output is locked
+     * and cursor hiding is not enabled and we are syncing the mouse
+     */
+
+    if (!optionGetScaleMouse () &&
+        (optionGetZoomMode () == EzoomOptions::ZoomModeSyncMouse &&
+	 optionGetHideOriginalMouse () &&
+	 !zooms.at (out).locked))
 	return;
 
     if (!cursorInfoSelected)
@@ -1235,7 +1217,8 @@ EZoomScreen::cursorZoomActive ()
 	updateCursor (&cursor);
     }
     if (canHideCursor && !cursorHidden &&
-	optionGetHideOriginalMouse ())
+	(optionGetHideOriginalMouse () ||
+	 zooms.at (out).locked))
     {
 	cursorHidden = true;
 	XFixesHideCursor (screen->dpy (), screen->root ());
@@ -1243,15 +1226,15 @@ EZoomScreen::cursorZoomActive ()
 }
 
 /* Set the zoom area
- * This is an interface for scripting. 
+ * This is an interface for scripting.
  * int32:x1: left x coordinate
  * int32:y1: top y coordinate
  * int32:x2: right x
- * int32:y2: bottom y 
+ * int32:y2: bottom y
  * x2 and y2 can be omitted to assume x1==x2+1 y1==y2+1
  * boolean:scale: True if we should modify the zoom level, false to just
  *                adjust the movement/translation.
- * boolean:restrain: True to warp the pointer so it's visible. 
+ * boolean:restrain: True to warp the pointer so it's visible.
  */
 bool
 EZoomScreen::setZoomAreaAction (CompAction         *action,
@@ -1260,7 +1243,7 @@ EZoomScreen::setZoomAreaAction (CompAction         *action,
 {
     int        x1, y1, x2, y2, out;
     bool       scale, restrain;
-    CompOutput *o; 
+    CompOutput *o;
 
     x1 = CompOption::getIntOptionNamed (options, "x1", -1);
     y1 = CompOption::getIntOptionNamed (options, "y1", -1);
@@ -1284,12 +1267,12 @@ EZoomScreen::setZoomAreaAction (CompAction         *action,
     setZoomArea (x1, y1, WIDTH, HEIGHT, false);
     o = &screen->outputDevs (). at(out);
     if (scale && WIDTH && HEIGHT)
-        setScaleBigger (out, (float) WIDTH / o->width (), 
+        setScaleBigger (out, (float) WIDTH / o->width (),
 		        (float) HEIGHT / o->height ());
 #undef WIDTH
 #undef HEIGHT
 	if (restrain)
-	    restrainCursor (out); 
+	    restrainCursor (out);
 
     toggleFunctions (true);
 
@@ -1334,7 +1317,7 @@ EZoomScreen::ensureVisibilityAction (CompAction         *action,
 #define WIDTH (x2 - x1)
 #define HEIGHT (y2 - y1)
     if (scale && WIDTH && HEIGHT)
-        setScaleBigger (out, (float) WIDTH / o->width (), 
+        setScaleBigger (out, (float) WIDTH / o->width (),
 		        (float) HEIGHT / o->height ());
 #undef WIDTH
 #undef HEIGHT
@@ -1370,11 +1353,10 @@ EZoomScreen::zoomBoxDeactivate (CompAction         *action,
 			       CompAction::State  state,
 			       CompOption::Vector options)
 {
-    int x, y, width, height;
-
     if (grabIndex)
     {
         int        out;
+	int	   x, y, width, height;
         CompOutput *o;
 
         screen->removeGrab (grabIndex, NULL);
@@ -1382,24 +1364,24 @@ EZoomScreen::zoomBoxDeactivate (CompAction         *action,
 
         if (pointerX < clickPos.x ())
         {
-	        box.setX (pointerX);
-	        box.setWidth (clickPos.x () - pointerX);
+	    box.setX (pointerX);
+	    box.setWidth (clickPos.x () - pointerX);
         }
         else
         {
-	        box.setWidth (pointerX - clickPos.x ());
+	    box.setWidth (pointerX - clickPos.x ());
         }
 
         if (pointerY < clickPos.y ())
         {
-	        box.setY (pointerY);
-	        box.setHeight (clickPos.y () - pointerY);
+	    box.setY (pointerY);
+	    box.setHeight (clickPos.y () - pointerY);
         }
         else
         {
-	        box.setHeight (pointerY - clickPos.y ());
+	    box.setHeight (pointerY - clickPos.y ());
         }
-        
+
         x = MIN (box.x1 (), box.x2 ());
         y = MIN (box.y1 (), box.y2 ());
         width = MAX (box.x1 (), box.x2 ()) - x;
@@ -1428,7 +1410,8 @@ EZoomScreen::zoomIn (CompAction         *action,
 {
     int out = screen->outputDeviceForPoint (pointerX, pointerY);
 
-    if (optionGetSyncMouse ()&& !isInMovement (out))
+    if (optionGetZoomMode () == EzoomOptions::ZoomModeSyncMouse &&
+	!isInMovement (out))
         setCenter (pointerX, pointerY, true);
 
     setScale (out,
@@ -1518,7 +1501,7 @@ EZoomScreen::zoomToWindow (CompAction         *action,
     height = w->height () + w->input ().top + w->input ().bottom;
     out = screen->outputDeviceForGeometry (w->geometry ());
     o = &screen->outputDevs ().at (out);
-    setScaleBigger (out, (float) width/o->width (), 
+    setScaleBigger (out, (float) width/o->width (),
 		    (float) height/o->height ());
     areaToWindow (w);
     toggleFunctions (true);
@@ -1590,8 +1573,8 @@ EZoomScreen::zoomFitWindowToZoom (CompAction         *action,
 			zooms.at (out).currentZoom -
 			(int) ((w->input ().top + w->input ().bottom)));
 
-    w->constrainNewWindowSize (xwc.width, 
-			       xwc.height, 
+    w->constrainNewWindowSize (xwc.width,
+			       xwc.height,
 			       &xwc.width,
 			       &xwc.height);
 
@@ -1702,7 +1685,7 @@ EZoomScreen::focusTrack (XEvent *event)
     w = screen->findWindow (event->xfocus.window);
     if (w == NULL || w->id () == screen->activeWindow ())
 	return;
- 
+
     if (time(NULL) - lastChange < optionGetFollowFocusDelay () ||
 	!optionGetFollowFocus ())
 	return;
@@ -1715,9 +1698,9 @@ EZoomScreen::focusTrack (XEvent *event)
     {
 	int width = w->width () + w->input ().left + w->input ().right;
 	int height = w->height () + w->input ().top + w->input ().bottom;
-	float scale = MAX ((float) width/screen->outputDevs ().at(out).width (), 
+	float scale = MAX ((float) width/screen->outputDevs ().at(out).width (),
 			   (float) height/screen->outputDevs ().at (out).height ());
-	if (scale > optionGetAutoscaleMin ()) 
+	if (scale > optionGetAutoscaleMin ())
 		setScale (out, scale);
     }
 
@@ -1789,20 +1772,23 @@ EZoomScreen::CursorTexture::CursorTexture () :
 void
 EZoomScreen::postLoad ()
 {
+    const CompPoint &m = pollHandle.getCurrentPosition ();
+    int         out = screen->outputDeviceForPoint (m.x (), m.y ());
+
     if (!grabbed)
 	return;
-    
+
     toggleFunctions (true);
-    
+
     if (!pollHandle.active ())
 	enableMousePolling ();
-	
+
     foreach (ZoomArea &za, zooms)
     {
 	grabbed |= (1 << za.output);
     }
 
-    cursorZoomActive ();
+    cursorZoomActive (out);
     updateCursor (&cursor);
 
     cScreen->damageScreen ();
@@ -1826,8 +1812,8 @@ EZoomScreen::EZoomScreen (CompScreen *screen) :
 
     int major, minor;
     unsigned int n;
-    fixesSupported = 
-	XFixesQueryExtension(screen->dpy (), 
+    fixesSupported =
+	XFixesQueryExtension(screen->dpy (),
 			     &fixesEventBase,
 			     &fixesErrorBase);
 
