@@ -2,7 +2,7 @@
  *
  * Compiz application switcher plugin
  *
- * staticswitcher.c
+ * staticswitcher.cpp
  *
  * Copyright : (C) 2008 by Danny Baumann
  * E-mail    : maniac@compiz-fusion.org
@@ -226,57 +226,10 @@ StaticSwitchScreen::handleSelectionChange (bool toNext, int nextIdx)
     moreAdjust = true;
 }
 
-bool
-StaticSwitchScreen::showPopup ()
-{
-    CompWindow *w = ::screen->findWindow (popupWindow);
-    if (w && (w->state () & CompWindowStateHiddenMask))
-    	w->show ();
-    else
-	XMapWindow (::screen->dpy (), popupWindow);
-
-    cScreen->damageScreen ();
-
-    popupDelayTimer.stop ();
-
-    return false;
-}
-
-Cursor
-StaticSwitchScreen::getCursor (bool mouseSelectOn)
-{
-    if (mouseSelectOn)
-	return ::screen->normalCursor ();
-
-    return ::screen->invisibleCursor ();
-}
-
 void
-StaticSwitchScreen::initiate (SwitchWindowSelection selection,
-			      bool                  shouldShowPopup)
+StaticSwitchScreen::createPopup ()
 {
-    bool noSwitchWindows;
-    bool newMouseSelect;
-
-    if (::screen->otherGrabExist ("switcher", "scale", "cube", 0))
-	return;
-
-    this->selection      = selection;
-    selectedWindow       = NULL;
-
-    noSwitchWindows = true;
-    foreach (CompWindow *w, ::screen->windows ())
-    {
-	if (StaticSwitchWindow::get (w)->isSwitchWin ())
-	{
-	    noSwitchWindows = false;
-	    break;
-	}
-    }
-    if (noSwitchWindows)
-	return;
-
-    if (!popupWindow && shouldShowPopup)
+    if (!popupWindow)
     {
 	Display		     *dpy = ::screen->dpy ();
 	XWMHints	     xwmh;
@@ -329,7 +282,65 @@ StaticSwitchScreen::initiate (SwitchWindowSelection selection,
 	::screen->setWindowProp (popupWindow, Atoms::winDesktop, 0xffffffff);
 
 	setSelectedWindowHint ();
+
+	updatePopupWindow ();
     }
+}
+
+bool
+StaticSwitchScreen::showPopup ()
+{
+    CompWindow *w;
+
+    /* Always checks for an existing popup */
+    createPopup ();
+
+    w = ::screen->findWindow (popupWindow);
+    if (w && (w->state () & CompWindowStateHiddenMask))
+    	w->show ();
+    else
+	XMapWindow (::screen->dpy (), popupWindow);
+
+    cScreen->damageScreen ();
+
+    popupDelayTimer.stop ();
+
+    return false;
+}
+
+Cursor
+StaticSwitchScreen::getCursor (bool mouseSelectOn)
+{
+    if (mouseSelectOn)
+	return ::screen->normalCursor ();
+
+    return ::screen->invisibleCursor ();
+}
+
+void
+StaticSwitchScreen::initiate (SwitchWindowSelection selection,
+			      bool                  shouldShowPopup)
+{
+    bool noSwitchWindows;
+    bool newMouseSelect;
+
+    if (::screen->otherGrabExist ("switcher", "scale", "cube", 0))
+	return;
+
+    this->selection      = selection;
+    selectedWindow       = NULL;
+
+    noSwitchWindows = true;
+    foreach (CompWindow *w, ::screen->windows ())
+    {
+	if (StaticSwitchWindow::get (w)->isSwitchWin ())
+	{
+	    noSwitchWindows = false;
+	    break;
+	}
+    }
+    if (noSwitchWindows)
+	return;
 
     newMouseSelect = optionGetMouseSelect () &&
     	             selection != Panels && shouldShowPopup;
@@ -349,7 +360,7 @@ StaticSwitchScreen::initiate (SwitchWindowSelection selection,
 
 	    createWindowList ();
 
-	    if (popupWindow && shouldShowPopup)
+	    if (shouldShowPopup)
 	    {
 		unsigned int delay;
 
@@ -371,6 +382,7 @@ StaticSwitchScreen::initiate (SwitchWindowSelection selection,
 		setSelectedWindowHint ();
 	    }
 
+	    lastActiveWindow = screen->activeWindow ();
 	    activateEvent (true);
 	}
 
@@ -448,6 +460,14 @@ switchTerminate (CompAction         *action,
 
 	ss->selectedWindow = NULL;
 
+	if (screen->activeWindow () != ss->lastActiveWindow)
+	{
+	    CompWindow *w = screen->findWindow (ss->lastActiveWindow);
+
+	    if (w)
+		w->moveInputFocusTo ();
+	}
+
 	ss->setSelectedWindowHint ();
 
 	ss->lastActiveNum = 0;
@@ -1143,7 +1163,6 @@ StaticSwitchWindow::glPaint (const GLWindowPaintAttrib &attrib,
 	float          px, py, pos;
 	int            count = sScreen->windows.size ();
 
-
 	CompWindow::Geometry &g = window->geometry ();
 
 	if (mask & PAINT_WINDOW_OCCLUSION_DETECTION_MASK ||
